"use strict";
var ErrorInterceptors_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorInterceptors = exports.PostgresErrors = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
var PostgresErrors;
(function (PostgresErrors) {
    PostgresErrors["OperatorDoesNotExist"] = "42883";
    PostgresErrors["InvalidTimestamp"] = "22007";
    PostgresErrors["InvalidType"] = "22P02";
    PostgresErrors["KeyConstraint"] = "23503";
    PostgresErrors["DuplicateKey"] = "23505";
    PostgresErrors["OutOfRange"] = "22003";
})(PostgresErrors = exports.PostgresErrors || (exports.PostgresErrors = {}));
let ErrorInterceptors = ErrorInterceptors_1 = class ErrorInterceptors {
    constructor() {
        this.logger = new common_1.Logger(ErrorInterceptors_1.name);
    }
    intercept(context, next) {
        return next.handle().pipe((0, rxjs_1.catchError)((error) => {
            if (Object.values(PostgresErrors).includes(error.code)) {
                return (0, rxjs_1.throwError)(() => this.preparePostgresError(context, error));
            }
            if (error instanceof common_1.HttpException) {
                return (0, rxjs_1.throwError)(() => this.prepareHttpError(context, error));
            }
            const preparedError = new common_1.HttpException({
                errors: [
                    {
                        detail: 'Internal server error',
                    },
                ],
            }, 500);
            preparedError.stack = error.stack;
            return (0, rxjs_1.throwError)(() => preparedError);
        }), (0, rxjs_1.tap)({
            error: (error) => {
                if ((error instanceof common_1.HttpException && error.getStatus() >= 400) ||
                    error.getStatus() < 500) {
                    this.logger.debug(error);
                }
                else {
                    this.logger.error(error);
                }
            },
        }));
    }
    prepareHttpError(context, error) {
        const response = error.getResponse();
        const status = error.getStatus();
        const errors = [];
        switch (typeof response.message) {
            case 'string':
                errors.push({ detail: response.message });
                break;
            case 'object':
                if (Array.isArray(response.message)) {
                    errors.push(...response.message);
                }
                else {
                    errors.push(response.message);
                }
                break;
            default:
                errors.push(response);
        }
        const preparedError = new common_1.HttpException({ status, errors }, status);
        preparedError.stack = error.stack;
        return preparedError;
    }
    preparePostgresError(context, error) {
        const request = context.switchToHttp().getRequest();
        const errors = [];
        let statusCode = 400;
        switch (error.code) {
            case PostgresErrors.InvalidTimestamp:
            case PostgresErrors.InvalidType: {
                Object.entries(request.query.filter || {}).forEach(([key, value]) => {
                    const test = typeof value === 'object' ? Object.values(value).pop() : value;
                    if (error.message.includes(`"${test}"`)) {
                        errors.push({
                            detail: `Filter param '${key}' has invalid type`,
                            source: {
                                parameter: "filter" /* QueryField.filter */,
                            },
                        });
                    }
                });
                break;
            }
            case PostgresErrors.DuplicateKey: {
                const matches = error.detail.match(/(?<=\().+?(?=\))/gm);
                errors.push({
                    source: {
                        pointer: `/data/attributes/${matches[0]}`,
                    },
                    detail: `Duplicate value '${matches[1]}' in the '${matches[0]}' attribute`,
                });
                statusCode = 409;
                break;
            }
            case PostgresErrors.OutOfRange: {
                const errorMsg = error.driverError.message;
                errors.push({
                    detail: errorMsg.charAt(0).toUpperCase() + errorMsg.slice(1),
                });
                break;
            }
            case PostgresErrors.KeyConstraint: {
                errors.push({
                    detail: 'You must clean entity relationships before this operation',
                });
                break;
            }
            case PostgresErrors.OperatorDoesNotExist: {
                errors.push({
                    detail: 'Filter params contain invalid operator',
                    source: {
                        parameter: "filter" /* QueryField.filter */,
                    },
                });
                break;
            }
        }
        const preparedError = new common_1.HttpException({ errors }, statusCode);
        preparedError.stack = error.stack;
        return preparedError;
    }
};
ErrorInterceptors = ErrorInterceptors_1 = tslib_1.__decorate([
    (0, common_1.Injectable)()
], ErrorInterceptors);
exports.ErrorInterceptors = ErrorInterceptors;
//# sourceMappingURL=error.interceptors.js.map